@@ -2,15 +2,14 @@ require 'pathname'
 
 module JSON
   class Schema
-
     attr_accessor :schema, :uri, :validator
 
-    def initialize(schema,uri,parent_validator=nil)
+    def initialize(schema, uri, parent_validator = nil)
       @schema = schema
       @uri = uri
 
       # If there is an ID on this schema, use it to generate the URI
-      if @schema['id'] && @schema['id'].kind_of?(String)
+      if @schema['id'].is_a?(String)
         temp_uri = JSON::Util::URI.parse(@schema['id'])
         if temp_uri.relative?
           temp_uri = uri.join(temp_uri)
@@ -20,13 +19,13 @@ module JSON
       @uri = JSON::Util::URI.strip_fragment(@uri)
 
       # If there is a $schema on this schema, use it to determine which validator to use
-      if @schema['$schema']
-        @validator = JSON::Validator.validator_for_uri(@schema['$schema'])
-      elsif parent_validator
-        @validator = parent_validator
-      else
-        @validator = JSON::Validator.default_validator
-      end
+      @validator = if @schema['$schema']
+                     JSON::Validator.validator_for_uri(@schema['$schema'])
+                   elsif parent_validator
+                     parent_validator
+                   else
+                     JSON::Validator.default_validator
+                   end
     end
 
     def validate(data, fragments, processor, options = {})
@@ -35,13 +34,13 @@ module JSON
 
     def self.stringify(schema)
       case schema
-      when Hash then
-        Hash[schema.map { |key, value| [key.to_s, stringify(schema[key])] }]
-      when Array then
+      when Hash
+        schema.map { |key, _value| [key.to_s, stringify(schema[key])] }.to_h
+      when Array
         schema.map do |schema_item|
           stringify(schema_item)
         end
-      when Symbol then
+      when Symbol
         schema.to_s
       else
         schema
